package com.example.service.impl;

import com.example.entity.auth.Account;
import com.example.mapper.UserMapper;
import com.example.service.AuthorizeService;
import jakarta.annotation.Resource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.mail.MailException;
import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.Optional;
import java.util.Random;
import java.util.concurrent.TimeUnit;

@Service
public class AuthorizeServiceImpl implements AuthorizeService {

    @Value("${spring.mail.username}")
    private String from;

    @Resource
    UserMapper mapper;

    @Resource
    MailSender mailSender;

    @Resource
    StringRedisTemplate template;

    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        if(username == null){
            throw new UsernameNotFoundException("用户名不能为空");
        }
        Account account = mapper.findAccountByNameOrEmail(username);
        if(account == null) {
            throw new UsernameNotFoundException("用户名或密码错误");
        }
        return User
                .withUsername(account.getUsername())
                .password(account.getPassword())
                .roles("user")
                .build();
    }
    /**
     * 1.先生成对应的验证码
     * 2.把邮箱和对应的验证码直接放到Redis里面（过期时间3分钟，如果此时重新要求发送邮件，
     * 当剩余时间小于2分钟时允许重新发送）
     * 3.发送验证码到指定邮箱
     * 4.如果发送失败，把redis里面的刚刚插入的删除
     * 5.用户在注册时，再从redis里面取出对应键值对，然后看验证码是否一致
     */
    @Override
    public String sendValidateEmail(String email, String sessionId, boolean hasAccount) {
        String key = "email：" + sessionId + ":" + email + ":" + hasAccount;
        if(Boolean.TRUE.equals(template.hasKey(key))){
            Long expire = Optional.ofNullable(template.getExpire(key, TimeUnit.SECONDS)).orElse(0L);
            if(expire > 120)
                return "请求频繁，请稍后再试";
        }
        Account account = mapper.findAccountByNameOrEmail(email);
        if(hasAccount && account == null)//重置密码时要求要有这个邮箱，但是数据库中没有
            return "此邮件未注册";
        if(!hasAccount && account != null)//注册时要求不能有重复的邮箱，但是数据库中有
            return "此邮箱以被注册";
        Random random = new Random();
        int code = random.nextInt(900000) + 100000;
        SimpleMailMessage message = new SimpleMailMessage();
        message.setFrom(from);
        message.setTo(email);
        message.setSubject("验证邮件");
        message.setText("验证码是：" + code);
        try{
            mailSender.send(message);
            template.opsForValue().set(key, String.valueOf(code), 3, TimeUnit.MINUTES);
            return null;
        } catch (MailException e){
            e.printStackTrace();
            return "邮件发送失败，请检查邮箱地址是否有效";
        }
    }

    @Override
    public String validateAndRegister(String username, String password, String email, String code, String sessionId) {
        String key = "email：" + sessionId + ":" + email + ":false";
        if(Boolean.TRUE.equals(template.hasKey(key))){
            String s = template.opsForValue().get(key);
            if(s == null)return "验证码失效，请重新获取";
            if(s.equals(code)){
                Account account = mapper.findAccountByNameOrEmail(username);
                if(account != null) return "此用户名已被使用";
                template.delete(key);
                password = encoder.encode(password);
                if (mapper.createAccount(email, username,password) > 0) {
                    return null;
                }else{
                    return "内部错误，请联系管理员";
                }
            }else {
                return "验证码错误，请检查后提交";
            }
        }else{
            return "请先获取验证码验证邮箱";
        }
    }

    @Override
    public String validateOnly(String email, String code, String sessionId) {
        String key = "email：" + sessionId + ":" + email + ":true";
        if(Boolean.TRUE.equals(template.hasKey(key))){
            String s = template.opsForValue().get(key);
            if(s == null)return "验证码失效，请重新获取";
            if(s.equals(code)){
                template.delete(key);
                return null;
            }else {
                return "验证码错误，请检查后提交";
            }
        }else{
            return "请先获取验证码验证邮箱";
        }
    }

    @Override
    public boolean resetPassword(String password, String email) {
        password = encoder.encode(password);
        return mapper.resetPasswordByEmail(password,email) > 0;
    }
}
